//+------------------------------------------------------------------+
//| MAAngle.mq4                                                      |
//| original jpkfox                                                  |
//| edited by dariuske                                               |
//| You can use this indicator to measure when the MA angle is       |
//| "near zero". AngleTreshold determines when the angle for the     |
//| EMA is "about zero": This is when the value is between           |
//| [-AngleTreshold, AngleTreshold] (or when the histogram is red).  |
//| MAMode : 0 = SMA, 1 = EMA, 2 = Smoothed, 3 = Weighted            |
//| MAPeriod: MA period                                              |
//| AngleTreshold: The angle value is "about zero" when it is        |
//| between the values [-AngleTreshold, AngleTreshold].              | 
//| StartMAShift: The starting point to calculate the                | 
//| angle. This is a shift value to the left from the                |
//| observation point. Should be StartEMAShift > EndEMAShift.        | 
//| StartMAShift: The ending point to calculate the                  |
//| angle. This is a shift value to the left from the                | 
//| observation point. Should be StartEMAShift > EndEMAShift.        |
//|                                                                  |
//| Modified by MrPip                                                |
//| Red for down                                                     |
//| Yellow for near zero                                             |
//| Green for up                                                     |
//| 10/15/05 MrPip                                                   |
//| Corrected problem with USDJPY and optimized code                 |
//| 10/23/05 Added other JPY crosses                                 |
//|                                                                  |
//|                                                                  | 
//|                                                                  | 
//| 12/01/07 Dariuske: Changed code for SMA50 (PhilNelSystem)        |
//| 18/01/07 Dariuske: Changed code for multiple mode MA's           |
//|                                                                  |
//| 27/10/2009 cja; tsd forum added MTF & Bars limit code            |                               
//+------------------------------------------------------------------+

#property copyright "jpkfox"
#property link "http://www.strategybuilderfx.com/forums/showthread.php?t=15274&page=1&pp=8"
//---- indicator settings
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 LimeGreen
#property indicator_color2 FireBrick 
#property indicator_color3 Yellow
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2


//---- indicator parameters
/*************************************************************************
PERIOD_M1   1
PERIOD_M5   5
PERIOD_M15  15
PERIOD_M30  30 
PERIOD_H1   60
PERIOD_H4   240
PERIOD_D1   1440
PERIOD_W1   10080
PERIOD_MN1  43200
You must use the numeric value of the timeframe that you want to use
when you set the TimeFrame' value with the indicator inputs.
---------------------------------------*/

extern int TimeFrame_Shift = 1;

extern int MaxBars=500;

//extern int    TimeFrame     = 60;//
extern int    MAMode        = 0;
extern int    MAPeriod      = 50;
extern int    Price         = 4;
extern double AngleTreshold = 0.25;
extern int    StartMAShift  = 2;
extern int    EndMAShift    = 0;
//----
int TimeFrame = 0;


//----


string TF1;

//---- indicator buffers
double UpBuffer[];
double DownBuffer[];
double ZeroBuffer[];
int tfs[9];

//+------------------------------------------------------------------+
//| Custom indicator initialization function |
//+------------------------------------------------------------------+
int init()
{
//---- 2 additional buffers are used for counting.
IndicatorBuffers(3);
//---- drawing settings
SetIndexStyle(0,DRAW_HISTOGRAM);
SetIndexStyle(1,DRAW_HISTOGRAM);
SetIndexStyle(2,DRAW_HISTOGRAM);

IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+2 );

//---- 3 indicator buffers mapping
if(!SetIndexBuffer(0,UpBuffer) &&
!SetIndexBuffer(1,DownBuffer) &&
!SetIndexBuffer(2,ZeroBuffer))
Print("cannot set indicator buffers!");

SetIndexLabel(0,"");
SetIndexLabel(1,"");
SetIndexLabel(2,"");

int itf = 0;
   
   tfs[0] = 1;
   tfs[1] = 5;
   tfs[2] = 15;
   tfs[3] = 30;
   tfs[4] = 60;
   tfs[5] = 240;
   tfs[6] = 1440;
   tfs[7] = 10080;
   tfs[8] = 43200;
   
   for( int i = 0; i < 9; i++ ) {
      if( Period() == tfs[i] )
         itf = i;
   }
   
   if( itf + TimeFrame_Shift > 8 ) {
      itf = Period();
      TimeFrame_Shift = 0;
   }
      
   
   TimeFrame = tfs[itf+TimeFrame_Shift];



//---- name for DataWindow and indicator subwindow label   
   switch(TimeFrame)
	{
		case 1:		TF1="M1";  break;
		case 5:		TF1="M5";  break;
		case 15:		TF1="M15"; break;
		case 30:		TF1="M30"; break;
		case 60:		TF1="H1";  break;
		case 240:	TF1="H4";  break;
		case 1440:	TF1="D1";  break;
		case 10080:	TF1="W1";  break;
		case 43200:	TF1="MN1"; break;
		default:	  {TimeFrame = Period(); init(); return(0);}
	}
   IndicatorShortName("MAAngle MTF  "+TF1+" ");
//---- initialization done
return(0);
}
//+------------------------------------------------------------------+
//| The angle for EMA |
//+------------------------------------------------------------------+
int start()
{
double fEndMA, fStartMA;
double fAngle, mFactor, dFactor;
int nLimit, i;
int nCountedBars;
double angle;
int ShiftDif;
string Sym;

if (MAMode >= 4) MAMode = 0;

if(EndMAShift >= StartMAShift)
{
Print("Error: EndMAShift >= StartMAShift");
StartMAShift = 6;
EndMAShift = 0; 
} 

nCountedBars = IndicatorCounted();

dFactor = 2*3.14159/180.0;
mFactor = 10000.0;
Sym = StringSubstr(Symbol(),3,3);
if (Sym == "JPY") mFactor = 100.0;
ShiftDif = StartMAShift-EndMAShift;
mFactor /= ShiftDif; 

//---- check for possible errors
     if(nCountedBars<0) return(-1);
//---- last counted bar will be recounted
     if(nCountedBars>0) nCountedBars--;
     nLimit=Bars-nCountedBars;
     nLimit=MathMax(nLimit,TimeFrame/Period());
     nLimit=MathMin(nLimit,MaxBars );  
//---- main loop
for(i=0; i<nLimit; i++)
{

 int y = iBarShift(NULL,TimeFrame,Time[i]);
 if (TimeFrame<Period()) TimeFrame=Period();
fEndMA=iMA(NULL,TimeFrame,MAPeriod,0,MAMode,Price,y+EndMAShift);
fStartMA=iMA(NULL,TimeFrame,MAPeriod,0,MAMode,Price,y+StartMAShift); 
 
// 10000.0 : Multiply by 10000 so that the fAngle is not too small
// for the indicator Window.
fAngle = mFactor * (fEndMA - fStartMA)/2.0;
//fAngle = MathArctan(fAngle)/dFactor;

DownBuffer[i] = 0.0;
UpBuffer[i] = 0.0;
ZeroBuffer[i] = 0.0;

if(fAngle > AngleTreshold)
UpBuffer[i] = fAngle;
else if (fAngle < -AngleTreshold)
DownBuffer[i] = fAngle;
else ZeroBuffer[i] = fAngle;
}

return(0);
}
//+------------------------------------------------------------------+



